use crate::{character::Character, dice::successes};

#[derive(Debug, PartialEq, Eq, Clone, Copy)]
pub enum EncounterType {
    Physical,
    Mental,
}

impl Character {
    pub fn attacks(&mut self, opponent: &mut Character) {
        let (margin, _succ, _fail, _succ_dice, fail_dice) = successes(
            self.dice_pool().max(0) as usize,
            (opponent.ac() - self.weapon_proficiency() as i64).clamp(1, 12),
        );

        let damage = self.compute_damage(margin); //(margin + self.weapon_proficiency()).max(0) * self.weapon.damage_dice;

        opponent.hp = opponent.hp - damage as i64;

        self.adjust_dice_pool((-fail_dice).min(0), EncounterType::Physical);

        log::trace!(
            "{} attacks {} getting a margin of {} and dealing {} damage.",
            self.name,
            opponent.name,
            margin,
            damage
        )
    }
}

pub fn make_them_fight(mut red: Vec<&mut Character>, mut blue: Vec<&mut Character>) -> Option<i64> {
    for character in red.iter_mut() {
        character.init_encounter(EncounterType::Physical);
    }

    for character in blue.iter_mut() {
        character.init_encounter(EncounterType::Physical);
    }

    let mut turn = 0;

    let mut dead_r = 0;
    let mut dead_b = 0;

    while red.iter().any(|c| !c.is_dead()) && blue.iter().any(|c| !c.is_dead()) {
        turn += 1;
        if turn > 15 {
            return None;
        }

        log::trace!("Turn {}", turn);
        log::trace!(
            "{:?}",
            red.iter()
                .map(|c| (c.name.to_owned(), c.hp))
                .collect::<Vec<(String, i64)>>()
        );
        log::trace!(
            "{:?}",
            blue.iter()
                .map(|c| (c.name.to_owned(), c.hp))
                .collect::<Vec<(String, i64)>>()
        );

        let mut init_r = 0;
        let mut init_b = 0;

        let mut target_r = 0;
        let mut target_b = 0;

        while init_r < red.len() || init_b < blue.len() {
            // Find next living red char
            log::trace!("Searching init");
            while init_r < red.len() && red[init_r].is_dead() {
                init_r += 1;
            }

            // If character is found
            if init_r < red.len() {
                // Find target
                log::trace!("Searching target");
                while blue[target_b].is_dead() {
                    target_b += 1;
                    target_b %= blue.len();
                }
                log::trace!("Target found");

                // Attack target
                red[init_r].attacks(blue[target_b]);

                if blue[target_b].is_dead() {
                    log::trace!("{} is dead.", blue[target_b].name);
                    dead_b += 1;
                }

                target_b += 1;
                target_b %= blue.len();
                init_r += 1;
            }

            // All blues are dead
            if dead_b == blue.len() {
                break;
            }

            // Find next living blue
            log::trace!("Searching init");
            while init_b < blue.len() && blue[init_b].is_dead() {
                init_b += 1;
            }

            // If found
            if init_b < blue.len() {
                log::trace!("Searching target");
                while red[target_r].hp <= 0 {
                    target_r += 1;
                    target_r %= red.len();
                }

                blue[init_b].attacks(red[target_r]);

                if red[target_r].is_dead() {
                    log::trace!("{} is dead.", red[target_r].name);
                    dead_r += 1;
                }

                target_r += 1;
                target_r %= red.len();
                init_b += 1;
            }

            if dead_r == red.len() {
                break;
            }
        }
    }

    Some(turn)
}

#[cfg(test)]
mod combat_tests {
    use crate::premade::{get_bob, get_drub};

    use super::make_them_fight;

    #[test_log::test]
    fn make_them_fight_debug() {
        let mut red = vec![get_bob(), get_bob()];
        red[1].name = "Bob 2".to_owned();
        let mut blue = vec![get_drub(), get_drub()];
        blue[1].name = "Drub 2".to_owned();

        make_them_fight(red.iter_mut().collect(), blue.iter_mut().collect());

        print!(
            "{:?}",
            red.into_iter()
                .map(|c| (c.name, c.hp))
                .collect::<Vec<(String, i64)>>()
        );
        print!(
            "{:?}",
            blue.into_iter()
                .map(|c| (c.name, c.hp))
                .collect::<Vec<(String, i64)>>()
        );
    }
}
